#!/bin/bash -e

# 服务器地址
api_server="board.bianbu.xyz:5000"

# 获取机器ID
get_machine_id() {
    # 尝试读取设备的序列号
    if cat /proc/device-tree/serial-number 2>/dev/null; then
        :
    else
        # 如果失败，错误消息
        echo "Error: Failed to retrieve machine ID." 1>&2
    fi
}

# 获取IP地址
get_ip() {
    # 查找第一个非本地回环网络接口的IPv4地址
    local active_interface=$(ip -4 route ls | grep default | grep -Po '(?<=dev )(\S+)' | head -1)
    if [[ -z "$active_interface" ]]; then
        echo "Error: No active network interface found." 1>&2
    fi

    local ip_address=$(ip addr show "$active_interface" | grep 'inet ' | awk '{print $2}' | cut -d/ -f1)
    if [[ -z "$ip_address" ]]; then
        echo "Error: No IP address found for interface  $active_interface." 1>&2
    fi
    echo "$ip_address"
}

# 调用API设置主机名
get_hostname() {
    local machine_id=$1
    # 替换为实际的API URL
    local api_url="http://$api_server/hostname?machine_id=${machine_id}"
    response=$(curl -s "${api_url}")
    # 使用 jq 解析 JSON 响应并提取主机名
    hostname=$(echo $response | jq -r '.hostname')
    echo $hostname
}

# 上传机器信息
upload_machine_info() {
    local machine_id=$1
    local ip=$2
    local kubelet_status=$(systemctl status kubelet | grep "Active:")

    # 替换为实际的API URL
    local api_url="http://$api_server/ping"

    # 构建JSON数据
    json_payload=$(
        cat <<EOF
{
"machine_id": "$machine_id",
"ip": "$ip",
"kubelet_status": "$kubelet_status"
}
EOF
    )

    response=$(curl -s -X POST "$api_url" \
        -H "Content-Type: application/json" \
        -d "$json_payload")
    # 使用 jq 解析 JSON 响应并提取主机名
    result=$(echo $response | jq -r '.success')
}

# 获取节点注册令牌
get_token() {
    # 替换为实际的API URL
    local api_url="http://$api_server/token"
    response=$(curl -s "${api_url}")
    # 使用 jq 解析 JSON 响应并提取主机名
    token=$(echo $response | jq -r '.token')
    echo $token
}

# 注册节点
register_node() {
    local token=$1
    echo "Registering node with token: ${token}"
    kubeadm join 10.0.50.53:6443 --token ${token} --discovery-token-ca-cert-hash sha256:c5f94386d87701f11b6819c8bee1411c14b61869b3a086fcf7a8364bbb938e49
}

# 主逻辑区域

# 获取机器ID和IP
machine_id=$(get_machine_id | tr -d '\0')
ip=$(get_ip)

# 检查机器ID和IP是否为空，如果为空，则打印错误信息并退出脚本
if [ -z "$machine_id" ] || [ -z "$ip" ]; then
    echo "Error: machine_id or ip is empty. Skip auto register node."
    exit 0
fi

echo "detect machine_id: $machine_id"
echo "detect ip: $ip"

# 获取并设置主机名
hostname=$(get_hostname "$machine_id")
# 检查hostname是否为空，如果为空，则打印错误信息并退出脚本
if [ -z "$hostname" ]; then
    echo "Error: hostname is empty. Skip auto register node."
    exit 0
fi

echo "Setting hostname to: ${hostname}"
hostnamectl hostname "${hostname}"

# 上传信息
upload_machine_info "$machine_id" "$ip"

# 获取令牌并注册节点
token=$(get_token)
# 检查hostname是否为空，如果为空，则打印错误信息并退出脚本
if [ -z "$token" ]; then
    echo "Error: token is empty. Skip auto register node."
    exit 0
fi

register_node "$token"
touch /etc/k8s-first-boot-init-done
echo "Initialization complete."
